<?php

declare(strict_types=1);

namespace Dedi\SyliusPayboxPlugin\Payum\Action;

use Dedi\SyliusPayboxPlugin\Payum\PayboxApi;
use Payum\Core\Action\ActionInterface;
use Payum\Core\ApiAwareInterface;
use Payum\Core\ApiAwareTrait;
use Payum\Core\Exception\RequestNotSupportedException;
use Payum\Core\Request\GetStatusInterface;
use Psr\Log\LoggerAwareInterface;
use Psr\Log\LoggerInterface;
use Sylius\Bundle\PayumBundle\Request\GetStatus;
use Sylius\Component\Core\Model\PaymentInterface;
use Symfony\Component\HttpFoundation\RequestStack;

final class StatusAction implements ActionInterface, ApiAwareInterface, LoggerAwareInterface
{
    use ApiAwareTrait;

    protected LoggerInterface $logger;

    public function __construct(private readonly RequestStack $requestStack)
    {
        $this->apiClass = PayboxApi::class;
    }

    public function execute($request)
    {
        RequestNotSupportedException::assertSupports($this, $request);

        /** @var PaymentInterface $payment */
        $payment = $request->getFirstModel();
        $details = $payment->getDetails();

        if (!isset($details['status'])) {
            $request->markNew();
        } elseif ($this->api->isAccepted($details['status'])) {
            $request->markCaptured();
        } elseif ($this->api->isFailed($details['status'])) {
            $request->markFailed();
        } else {
            $request->markCanceled();
        }

        $query = $this->requestStack->getCurrentRequest()->getQueryString();
        $this->logger->info(sprintf('[%s] %s', PayboxApi::CODE, self::class), ['response' => $query, 'payment' => $payment->getDetails()]);
    }

    public function setLogger(LoggerInterface $logger)
    {
        $this->logger = $logger;
    }

    public function supports($request)
    {
        return
            $request instanceof GetStatusInterface &&
            $request instanceof GetStatus &&
            $request->getFirstModel() instanceof PaymentInterface
        ;
    }
}
